{
 "cells": [
  {
   "cell_type": "markdown",
   "id": "7ba80f53-715b-4610-bb63-e5943ee809c4",
   "metadata": {},
   "source": [
    "# Networks with Altair\n",
    "\n",
    "## Dynamic Visualization\n",
    "\n",
    "Dynamic visualization responds to the viewer's inputs. In dynamic visualizations, you can click on things, move nodes around, and get more information about the network in various ways. All the principles of static visualization described on the previous page also apply to dynamic visualization. But because you can more easily show node movement and create new arrangements in a dynamic visualization, you can typically include a lot more information in a single visualization. The key to a dynamic visualization is that it allows someone to explore the network on their own. This makes it both easier and harder to communicate an argument with a dynamic visualization. There are more tools at your disposal, but the viewer also has considerable freedom.\n",
    "\n",
    "## Using `nx_altair`\n",
    "\n",
    "In the Data Basics section, you saw how to make interactive visualizations with Altair. There is a plugin for this library, `nx_altair`, that allows you to use Altair's syntax to create interactive network visualizations.\n",
    "\n",
    "Remember that before you can use a new library, you must install it with `!pip install nx_altair`, but you only need to do this one time.\n",
    "\n",
    "```{seealso}\n",
    "Some documentation and guidance for nx_altair can be found on its [Github page](https://github.com/Zsailer/nx_altair).\n",
    "```\n",
    "\n",
    "You'll use nx_altair in concert with Altair and NetworkX:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "id": "0296837a-b2fb-4e78-91f4-68a735718780",
   "metadata": {},
   "outputs": [
    {
     "name": "stdout",
     "output_type": "stream",
     "text": [
      "Graph named \"Zachary's Karate Club\" with 34 nodes and 78 edges\n"
     ]
    }
   ],
   "source": [
    "# Necessary libraries\n",
    "import networkx as nx\n",
    "import pandas as pd\n",
    "import altair as alt\n",
    "import nx_altair as nxa\n",
    "\n",
    "# Read Karate Club data as example\n",
    "G = nx.karate_club_graph()\n",
    "print(G)"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "1b29487b-d800-46f2-b10d-7656bf1d7dd4",
   "metadata": {},
   "source": [
    "You can make your network visualization more complex by first adding some attributes:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "id": "4bd19f68-0098-4ff6-bcbe-b36866300f18",
   "metadata": {
    "tags": []
   },
   "outputs": [],
   "source": [
    "degree = nx.degree_centrality(G) # Add degree centrality\n",
    "betweenness = nx.betweenness_centrality(G) # Add betweenness centrality\n",
    "# Run louvain community detection\n",
    "from networkx.algorithms.community import louvain_communities\n",
    "louvain = louvain_communities(G, seed=42)\n",
    "louvain = {g:str(i) for i,group in enumerate(louvain) for g in group}\n",
    "\n",
    "# Add all attributes to graph\n",
    "nx.set_node_attributes(G, degree, 'degree_centrality')\n",
    "nx.set_node_attributes(G, betweenness, 'betweenness_centrality')\n",
    "nx.set_node_attributes(G, louvain, \"louvain_partition\")"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "d52e6481-2f80-400e-8508-0dcc36e8a66d",
   "metadata": {},
   "source": [
    "## Creating a basic nx_altair graph\n",
    "\n",
    "The nx_altair plugin allows you to treat node and edge attributes just as you would columns in a tabular dataset (i.e. you can access them using quoted strings).\n",
    "\n",
    "```{seealso}\n",
    "`nx_altair` uses all the same attributes and options as the `draw_networkx` function in NetworkX. You can look at the [NetworkX documentation](https://networkx.org/documentation/stable/reference/generated/networkx.drawing.nx_pylab.draw_networkx.html) for a full list of what you can do with these options.\n",
    "```\n",
    "\n",
    "Before you can draw a NetworkX graph, you have to select a layout. There are lots of [layout options](https://networkx.org/documentation/stable/reference/drawing.html#module-networkx.drawing.layout), and the `spring_layout()` function will give you a basic force-directed network:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "id": "844cb6f9-06ee-489e-8450-da923677d805",
   "metadata": {},
   "outputs": [],
   "source": [
    "pos = nx.spring_layout(G) \n",
    "# You can customize the position in many ways!\n",
    "# This is only a starting point."
   ]
  },
  {
   "cell_type": "markdown",
   "id": "2cc31373-225e-429d-ac35-a605dd70a356",
   "metadata": {},
   "source": [
    "Once you have a layout, you can create a simple example of the `nx_altair` syntax with some of the available options:"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "id": "41caab1d-6099-4974-9c1a-0a6f1bd26cd3",
   "metadata": {},
   "outputs": [],
   "source": [
    "# As with other altair charts, you must sometimes disable max rows\n",
    "alt.data_transformers.disable_max_rows()\n",
    "viz = nxa.draw_networkx(G, \n",
    "                        pos=pos, \n",
    "                        node_color='louvain_partition', \n",
    "                        node_size='degree_centrality',\n",
    "                        node_tooltip=['degree_centrality','betweenness_centrality']\n",
    "                       )"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "id": "da242de0-c68c-40d9-874c-68c7e40be640",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-0056f57463d94eddbf7187e0084b1fa7.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-0056f57463d94eddbf7187e0084b1fa7.vega-embed details,\n",
       "  #altair-viz-0056f57463d94eddbf7187e0084b1fa7.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-0056f57463d94eddbf7187e0084b1fa7\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-0056f57463d94eddbf7187e0084b1fa7\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-0056f57463d94eddbf7187e0084b1fa7\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.16.3?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.16.3\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"layer\": [{\"data\": {\"name\": \"data-68c5fc4a60be90d486fe214af3b8cb49\"}, \"mark\": {\"type\": \"line\", \"color\": \"black\", \"opacity\": 1, \"strokeWidth\": 1}, \"encoding\": {\"detail\": {\"field\": \"edge\", \"type\": \"quantitative\"}, \"x\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false, \"title\": \"\"}, \"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false, \"title\": \"\"}, \"field\": \"y\", \"type\": \"quantitative\"}}, \"name\": \"view_1\"}, {\"data\": {\"name\": \"data-9dd8291c49693efa9806d078391fedf1\"}, \"mark\": {\"type\": \"point\", \"opacity\": 1, \"strokeWidth\": 1.0}, \"encoding\": {\"fill\": {\"field\": \"louvain_partition\", \"type\": \"nominal\"}, \"size\": {\"field\": \"degree_centrality\", \"legend\": null, \"type\": \"quantitative\"}, \"tooltip\": [{\"field\": \"degree_centrality\", \"type\": \"quantitative\"}, {\"field\": \"betweenness_centrality\", \"type\": \"quantitative\"}], \"x\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false}, \"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false}, \"field\": \"y\", \"type\": \"quantitative\"}}}], \"params\": [{\"name\": \"param_1\", \"select\": {\"type\": \"interval\", \"encodings\": [\"x\", \"y\"]}, \"bind\": \"scales\", \"views\": [\"view_1\"]}], \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.16.3.json\", \"datasets\": {\"data-68c5fc4a60be90d486fe214af3b8cb49\": [{\"target\": 1, \"weight\": 4, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 1], \"edge\": 0, \"source\": 0}, {\"target\": 1, \"weight\": 4, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [0, 1], \"edge\": 0, \"source\": 0}, {\"target\": 2, \"weight\": 5, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 2], \"edge\": 1, \"source\": 0}, {\"target\": 2, \"weight\": 5, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [0, 2], \"edge\": 1, \"source\": 0}, {\"target\": 3, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 3], \"edge\": 2, \"source\": 0}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [0, 3], \"edge\": 2, \"source\": 0}, {\"target\": 4, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 4], \"edge\": 3, \"source\": 0}, {\"target\": 4, \"weight\": 3, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [0, 4], \"edge\": 3, \"source\": 0}, {\"target\": 5, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 5], \"edge\": 4, \"source\": 0}, {\"target\": 5, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [0, 5], \"edge\": 4, \"source\": 0}, {\"target\": 6, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 6], \"edge\": 5, \"source\": 0}, {\"target\": 6, \"weight\": 3, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [0, 6], \"edge\": 5, \"source\": 0}, {\"target\": 7, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 7], \"edge\": 6, \"source\": 0}, {\"target\": 7, \"weight\": 2, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [0, 7], \"edge\": 6, \"source\": 0}, {\"target\": 8, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 8], \"edge\": 7, \"source\": 0}, {\"target\": 8, \"weight\": 2, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [0, 8], \"edge\": 7, \"source\": 0}, {\"target\": 10, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 10], \"edge\": 8, \"source\": 0}, {\"target\": 10, \"weight\": 2, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [0, 10], \"edge\": 8, \"source\": 0}, {\"target\": 11, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 11], \"edge\": 9, \"source\": 0}, {\"target\": 11, \"weight\": 3, \"y\": 0.597088045182672, \"x\": 0.007541601265146356, \"pair\": [0, 11], \"edge\": 9, \"source\": 0}, {\"target\": 12, \"weight\": 1, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 12], \"edge\": 10, \"source\": 0}, {\"target\": 12, \"weight\": 1, \"y\": 0.3853633418034468, \"x\": 0.6506086687459723, \"pair\": [0, 12], \"edge\": 10, \"source\": 0}, {\"target\": 13, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 13], \"edge\": 11, \"source\": 0}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [0, 13], \"edge\": 11, \"source\": 0}, {\"target\": 17, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 17], \"edge\": 12, \"source\": 0}, {\"target\": 17, \"weight\": 2, \"y\": 0.21975436759374803, \"x\": 0.6257949010031251, \"pair\": [0, 17], \"edge\": 12, \"source\": 0}, {\"target\": 19, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 19], \"edge\": 13, \"source\": 0}, {\"target\": 19, \"weight\": 2, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [0, 19], \"edge\": 13, \"source\": 0}, {\"target\": 21, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 21], \"edge\": 14, \"source\": 0}, {\"target\": 21, \"weight\": 2, \"y\": 0.3912476160151856, \"x\": 0.4663145131865174, \"pair\": [0, 21], \"edge\": 14, \"source\": 0}, {\"target\": 31, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 31], \"edge\": 15, \"source\": 0}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [0, 31], \"edge\": 15, \"source\": 0}, {\"target\": 2, \"weight\": 6, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 2], \"edge\": 16, \"source\": 1}, {\"target\": 2, \"weight\": 6, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [1, 2], \"edge\": 16, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 3], \"edge\": 17, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [1, 3], \"edge\": 17, \"source\": 1}, {\"target\": 7, \"weight\": 4, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 7], \"edge\": 18, \"source\": 1}, {\"target\": 7, \"weight\": 4, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [1, 7], \"edge\": 18, \"source\": 1}, {\"target\": 13, \"weight\": 5, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 13], \"edge\": 19, \"source\": 1}, {\"target\": 13, \"weight\": 5, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [1, 13], \"edge\": 19, \"source\": 1}, {\"target\": 17, \"weight\": 1, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 17], \"edge\": 20, \"source\": 1}, {\"target\": 17, \"weight\": 1, \"y\": 0.21975436759374803, \"x\": 0.6257949010031251, \"pair\": [1, 17], \"edge\": 20, \"source\": 1}, {\"target\": 19, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 19], \"edge\": 21, \"source\": 1}, {\"target\": 19, \"weight\": 2, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [1, 19], \"edge\": 21, \"source\": 1}, {\"target\": 21, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 21], \"edge\": 22, \"source\": 1}, {\"target\": 21, \"weight\": 2, \"y\": 0.3912476160151856, \"x\": 0.4663145131865174, \"pair\": [1, 21], \"edge\": 22, \"source\": 1}, {\"target\": 30, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 30], \"edge\": 23, \"source\": 1}, {\"target\": 30, \"weight\": 2, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [1, 30], \"edge\": 23, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 3], \"edge\": 24, \"source\": 2}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [2, 3], \"edge\": 24, \"source\": 2}, {\"target\": 7, \"weight\": 4, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 7], \"edge\": 25, \"source\": 2}, {\"target\": 7, \"weight\": 4, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [2, 7], \"edge\": 25, \"source\": 2}, {\"target\": 8, \"weight\": 5, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 8], \"edge\": 26, \"source\": 2}, {\"target\": 8, \"weight\": 5, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [2, 8], \"edge\": 26, \"source\": 2}, {\"target\": 9, \"weight\": 1, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 9], \"edge\": 27, \"source\": 2}, {\"target\": 9, \"weight\": 1, \"y\": -0.5999087969351852, \"x\": -0.04073657086420715, \"pair\": [2, 9], \"edge\": 27, \"source\": 2}, {\"target\": 13, \"weight\": 3, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 13], \"edge\": 28, \"source\": 2}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [2, 13], \"edge\": 28, \"source\": 2}, {\"target\": 27, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 27], \"edge\": 29, \"source\": 2}, {\"target\": 27, \"weight\": 2, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [2, 27], \"edge\": 29, \"source\": 2}, {\"target\": 28, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 28], \"edge\": 30, \"source\": 2}, {\"target\": 28, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [2, 28], \"edge\": 30, \"source\": 2}, {\"target\": 32, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 32], \"edge\": 31, \"source\": 2}, {\"target\": 32, \"weight\": 2, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [2, 32], \"edge\": 31, \"source\": 2}, {\"target\": 7, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 7], \"edge\": 32, \"source\": 3}, {\"target\": 7, \"weight\": 3, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [3, 7], \"edge\": 32, \"source\": 3}, {\"target\": 12, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 12], \"edge\": 33, \"source\": 3}, {\"target\": 12, \"weight\": 3, \"y\": 0.3853633418034468, \"x\": 0.6506086687459723, \"pair\": [3, 12], \"edge\": 33, \"source\": 3}, {\"target\": 13, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 13], \"edge\": 34, \"source\": 3}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [3, 13], \"edge\": 34, \"source\": 3}, {\"target\": 6, \"weight\": 2, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [4, 6], \"edge\": 35, \"source\": 4}, {\"target\": 6, \"weight\": 2, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [4, 6], \"edge\": 35, \"source\": 4}, {\"target\": 10, \"weight\": 3, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [4, 10], \"edge\": 36, \"source\": 4}, {\"target\": 10, \"weight\": 3, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [4, 10], \"edge\": 36, \"source\": 4}, {\"target\": 6, \"weight\": 5, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 6], \"edge\": 37, \"source\": 5}, {\"target\": 6, \"weight\": 5, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [5, 6], \"edge\": 37, \"source\": 5}, {\"target\": 10, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 10], \"edge\": 38, \"source\": 5}, {\"target\": 10, \"weight\": 3, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [5, 10], \"edge\": 38, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 16], \"edge\": 39, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 1.0, \"x\": 0.46353845653581993, \"pair\": [5, 16], \"edge\": 39, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [6, 16], \"edge\": 40, \"source\": 6}, {\"target\": 16, \"weight\": 3, \"y\": 1.0, \"x\": 0.46353845653581993, \"pair\": [6, 16], \"edge\": 40, \"source\": 6}, {\"target\": 30, \"weight\": 3, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 30], \"edge\": 41, \"source\": 8}, {\"target\": 30, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [8, 30], \"edge\": 41, \"source\": 8}, {\"target\": 32, \"weight\": 3, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 32], \"edge\": 42, \"source\": 8}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [8, 32], \"edge\": 42, \"source\": 8}, {\"target\": 33, \"weight\": 4, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 33], \"edge\": 43, \"source\": 8}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [8, 33], \"edge\": 43, \"source\": 8}, {\"target\": 33, \"weight\": 2, \"y\": -0.5999087969351852, \"x\": -0.04073657086420715, \"pair\": [9, 33], \"edge\": 44, \"source\": 9}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [9, 33], \"edge\": 44, \"source\": 9}, {\"target\": 33, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [13, 33], \"edge\": 45, \"source\": 13}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [13, 33], \"edge\": 45, \"source\": 13}, {\"target\": 32, \"weight\": 3, \"y\": -0.07645960350272783, \"x\": -0.5860519476705567, \"pair\": [14, 32], \"edge\": 46, \"source\": 14}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [14, 32], \"edge\": 46, \"source\": 14}, {\"target\": 33, \"weight\": 2, \"y\": -0.07645960350272783, \"x\": -0.5860519476705567, \"pair\": [14, 33], \"edge\": 47, \"source\": 14}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [14, 33], \"edge\": 47, \"source\": 14}, {\"target\": 32, \"weight\": 3, \"y\": -0.17961314191255023, \"x\": -0.5103533762523353, \"pair\": [15, 32], \"edge\": 48, \"source\": 15}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [15, 32], \"edge\": 48, \"source\": 15}, {\"target\": 33, \"weight\": 4, \"y\": -0.17961314191255023, \"x\": -0.5103533762523353, \"pair\": [15, 33], \"edge\": 49, \"source\": 15}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [15, 33], \"edge\": 49, \"source\": 15}, {\"target\": 32, \"weight\": 1, \"y\": -0.5082714987382673, \"x\": -0.6545540664183989, \"pair\": [18, 32], \"edge\": 50, \"source\": 18}, {\"target\": 32, \"weight\": 1, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [18, 32], \"edge\": 50, \"source\": 18}, {\"target\": 33, \"weight\": 2, \"y\": -0.5082714987382673, \"x\": -0.6545540664183989, \"pair\": [18, 33], \"edge\": 51, \"source\": 18}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [18, 33], \"edge\": 51, \"source\": 18}, {\"target\": 33, \"weight\": 1, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [19, 33], \"edge\": 52, \"source\": 19}, {\"target\": 33, \"weight\": 1, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [19, 33], \"edge\": 52, \"source\": 19}, {\"target\": 32, \"weight\": 3, \"y\": -0.6060332323179136, \"x\": -0.4421316341277511, \"pair\": [20, 32], \"edge\": 53, \"source\": 20}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [20, 32], \"edge\": 53, \"source\": 20}, {\"target\": 33, \"weight\": 1, \"y\": -0.6060332323179136, \"x\": -0.4421316341277511, \"pair\": [20, 33], \"edge\": 54, \"source\": 20}, {\"target\": 33, \"weight\": 1, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [20, 33], \"edge\": 54, \"source\": 20}, {\"target\": 32, \"weight\": 2, \"y\": -0.4752818193287819, \"x\": -0.4901750606421922, \"pair\": [22, 32], \"edge\": 55, \"source\": 22}, {\"target\": 32, \"weight\": 2, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [22, 32], \"edge\": 55, \"source\": 22}, {\"target\": 33, \"weight\": 3, \"y\": -0.4752818193287819, \"x\": -0.4901750606421922, \"pair\": [22, 33], \"edge\": 56, \"source\": 22}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [22, 33], \"edge\": 56, \"source\": 22}, {\"target\": 25, \"weight\": 5, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 25], \"edge\": 57, \"source\": 23}, {\"target\": 25, \"weight\": 5, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [23, 25], \"edge\": 57, \"source\": 23}, {\"target\": 27, \"weight\": 4, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 27], \"edge\": 58, \"source\": 23}, {\"target\": 27, \"weight\": 4, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [23, 27], \"edge\": 58, \"source\": 23}, {\"target\": 29, \"weight\": 3, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 29], \"edge\": 59, \"source\": 23}, {\"target\": 29, \"weight\": 3, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [23, 29], \"edge\": 59, \"source\": 23}, {\"target\": 32, \"weight\": 5, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 32], \"edge\": 60, \"source\": 23}, {\"target\": 32, \"weight\": 5, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [23, 32], \"edge\": 60, \"source\": 23}, {\"target\": 33, \"weight\": 4, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 33], \"edge\": 61, \"source\": 23}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [23, 33], \"edge\": 61, \"source\": 23}, {\"target\": 25, \"weight\": 2, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 25], \"edge\": 62, \"source\": 24}, {\"target\": 25, \"weight\": 2, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [24, 25], \"edge\": 62, \"source\": 24}, {\"target\": 27, \"weight\": 3, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 27], \"edge\": 63, \"source\": 24}, {\"target\": 27, \"weight\": 3, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [24, 27], \"edge\": 63, \"source\": 24}, {\"target\": 31, \"weight\": 2, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 31], \"edge\": 64, \"source\": 24}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [24, 31], \"edge\": 64, \"source\": 24}, {\"target\": 31, \"weight\": 7, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [25, 31], \"edge\": 65, \"source\": 25}, {\"target\": 31, \"weight\": 7, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [25, 31], \"edge\": 65, \"source\": 25}, {\"target\": 29, \"weight\": 4, \"y\": -0.31531373964572107, \"x\": -0.715204123907239, \"pair\": [26, 29], \"edge\": 66, \"source\": 26}, {\"target\": 29, \"weight\": 4, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [26, 29], \"edge\": 66, \"source\": 26}, {\"target\": 33, \"weight\": 2, \"y\": -0.31531373964572107, \"x\": -0.715204123907239, \"pair\": [26, 33], \"edge\": 67, \"source\": 26}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [26, 33], \"edge\": 67, \"source\": 26}, {\"target\": 33, \"weight\": 4, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [27, 33], \"edge\": 68, \"source\": 27}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [27, 33], \"edge\": 68, \"source\": 27}, {\"target\": 31, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [28, 31], \"edge\": 69, \"source\": 28}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [28, 31], \"edge\": 69, \"source\": 28}, {\"target\": 33, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [28, 33], \"edge\": 70, \"source\": 28}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [28, 33], \"edge\": 70, \"source\": 28}, {\"target\": 32, \"weight\": 4, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [29, 32], \"edge\": 71, \"source\": 29}, {\"target\": 32, \"weight\": 4, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [29, 32], \"edge\": 71, \"source\": 29}, {\"target\": 33, \"weight\": 2, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [29, 33], \"edge\": 72, \"source\": 29}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [29, 33], \"edge\": 72, \"source\": 29}, {\"target\": 32, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [30, 32], \"edge\": 73, \"source\": 30}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [30, 32], \"edge\": 73, \"source\": 30}, {\"target\": 33, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [30, 33], \"edge\": 74, \"source\": 30}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [30, 33], \"edge\": 74, \"source\": 30}, {\"target\": 32, \"weight\": 4, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [31, 32], \"edge\": 75, \"source\": 31}, {\"target\": 32, \"weight\": 4, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [31, 32], \"edge\": 75, \"source\": 31}, {\"target\": 33, \"weight\": 4, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [31, 33], \"edge\": 76, \"source\": 31}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [31, 33], \"edge\": 76, \"source\": 31}, {\"target\": 33, \"weight\": 5, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [32, 33], \"edge\": 77, \"source\": 32}, {\"target\": 33, \"weight\": 5, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [32, 33], \"edge\": 77, \"source\": 32}], \"data-9dd8291c49693efa9806d078391fedf1\": [{\"y\": 0.3097536305922184, \"degree_centrality\": 0.48484848484848486, \"x\": 0.22503121822115288, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.43763528138528146}, {\"y\": 0.13856825943954948, \"degree_centrality\": 0.2727272727272727, \"x\": 0.22132660602478407, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.053936688311688304}, {\"y\": -0.026723682993196735, \"degree_centrality\": 0.30303030303030304, \"x\": 0.11712199222035433, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.14365680615680618}, {\"y\": 0.20274363377868745, \"degree_centrality\": 0.18181818181818182, \"x\": 0.3613046705731705, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.011909271284271283}, {\"y\": 0.6247825385327798, \"degree_centrality\": 0.09090909090909091, \"x\": 0.26353120038140965, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0006313131313131313}, {\"y\": 0.760957365547892, \"degree_centrality\": 0.12121212121212122, \"x\": 0.3348488047284244, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.02998737373737374}, {\"y\": 0.7112806407924765, \"degree_centrality\": 0.12121212121212122, \"x\": 0.42635535723687124, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.029987373737373736}, {\"y\": 0.07616207795941865, \"degree_centrality\": 0.12121212121212122, \"x\": 0.36705456566283584, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.0}, {\"y\": -0.030123270746296296, \"degree_centrality\": 0.15151515151515152, \"x\": -0.09506079209675, \"club\": \"Mr. Hi\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.05592682780182781}, {\"y\": -0.5999087969351852, \"degree_centrality\": 0.06060606060606061, \"x\": -0.04073657086420715, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0008477633477633478}, {\"y\": 0.7550530935308861, \"degree_centrality\": 0.09090909090909091, \"x\": 0.178048351734203, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0006313131313131313}, {\"y\": 0.597088045182672, \"degree_centrality\": 0.030303030303030304, \"x\": 0.007541601265146356, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": 0.3853633418034468, \"degree_centrality\": 0.06060606060606061, \"x\": 0.6506086687459723, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.0}, {\"y\": 0.09163458388764495, \"degree_centrality\": 0.15151515151515152, \"x\": 0.062256995703755307, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.04586339586339586}, {\"y\": -0.07645960350272783, \"degree_centrality\": 0.06060606060606061, \"x\": -0.5860519476705567, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.17961314191255023, \"degree_centrality\": 0.06060606060606061, \"x\": -0.5103533762523353, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": 1.0, \"degree_centrality\": 0.06060606060606061, \"x\": 0.46353845653581993, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": 0.21975436759374803, \"degree_centrality\": 0.06060606060606061, \"x\": 0.6257949010031251, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": -0.5082714987382673, \"degree_centrality\": 0.06060606060606061, \"x\": -0.6545540664183989, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.05620242556660995, \"degree_centrality\": 0.09090909090909091, \"x\": 0.27260396115187274, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.03247504810004811}, {\"y\": -0.6060332323179136, \"degree_centrality\": 0.06060606060606061, \"x\": -0.4421316341277511, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": 0.3912476160151856, \"degree_centrality\": 0.06060606060606061, \"x\": 0.4663145131865174, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": -0.4752818193287819, \"degree_centrality\": 0.06060606060606061, \"x\": -0.4901750606421922, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.44048234585790097, \"degree_centrality\": 0.15151515151515152, \"x\": -0.2700696987833476, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.017613636363636363}, {\"y\": -0.6089417486546188, \"degree_centrality\": 0.09090909090909091, \"x\": 0.11368659554436691, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0022095959595959595}, {\"y\": -0.49977953574607614, \"degree_centrality\": 0.09090909090909091, \"x\": -0.08287243910457731, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0038404882154882154}, {\"y\": -0.31531373964572107, \"degree_centrality\": 0.06060606060606061, \"x\": -0.715204123907239, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.3938248407142819, \"degree_centrality\": 0.12121212121212122, \"x\": -0.05031006926735855, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.02233345358345358}, {\"y\": -0.3218466214147138, \"degree_centrality\": 0.09090909090909091, \"x\": 0.1326826127027591, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0017947330447330447}, {\"y\": -0.3438316635346198, \"degree_centrality\": 0.12121212121212122, \"x\": -0.5228689570526862, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0029220779220779218}, {\"y\": -0.00897933918312558, \"degree_centrality\": 0.12121212121212122, \"x\": -0.2085724152806764, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.014411976911976909}, {\"y\": -0.26271885859567407, \"degree_centrality\": 0.18181818181818182, \"x\": -0.03777649743004404, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.13827561327561325}, {\"y\": -0.25186155455325504, \"degree_centrality\": 0.36363636363636365, \"x\": -0.32679233890906567, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.145247113997114}, {\"y\": -0.2581914747150901, \"degree_centrality\": 0.5151515151515151, \"x\": -0.25612108481535745, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.30407497594997596}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.LayerChart(...)"
      ]
     },
     "execution_count": 5,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "# Use .interactive() to make your plot interactive\n",
    "viz.interactive()"
   ]
  },
  {
   "cell_type": "markdown",
   "id": "48e5afbe-fd25-4939-a5cd-ebedd2c16ca9",
   "metadata": {},
   "source": [
    "## Adding additional charts\n",
    "\n",
    "Because you're using Altair to make your chart, in addition to adding zooming interactivity, you can also add additional explanatory charts. For instance, you can make a bar plot to show the number of nodes in each partition, and add that to your network visualization."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "id": "24093f90-1dfb-47ab-85f0-b33b448dc8b7",
   "metadata": {},
   "outputs": [
    {
     "data": {
      "text/html": [
       "\n",
       "<style>\n",
       "  #altair-viz-1da3899da8b94985bb24fb827bc417b4.vega-embed {\n",
       "    width: 100%;\n",
       "    display: flex;\n",
       "  }\n",
       "\n",
       "  #altair-viz-1da3899da8b94985bb24fb827bc417b4.vega-embed details,\n",
       "  #altair-viz-1da3899da8b94985bb24fb827bc417b4.vega-embed details summary {\n",
       "    position: relative;\n",
       "  }\n",
       "</style>\n",
       "<div id=\"altair-viz-1da3899da8b94985bb24fb827bc417b4\"></div>\n",
       "<script type=\"text/javascript\">\n",
       "  var VEGA_DEBUG = (typeof VEGA_DEBUG == \"undefined\") ? {} : VEGA_DEBUG;\n",
       "  (function(spec, embedOpt){\n",
       "    let outputDiv = document.currentScript.previousElementSibling;\n",
       "    if (outputDiv.id !== \"altair-viz-1da3899da8b94985bb24fb827bc417b4\") {\n",
       "      outputDiv = document.getElementById(\"altair-viz-1da3899da8b94985bb24fb827bc417b4\");\n",
       "    }\n",
       "    const paths = {\n",
       "      \"vega\": \"https://cdn.jsdelivr.net/npm/vega@5?noext\",\n",
       "      \"vega-lib\": \"https://cdn.jsdelivr.net/npm/vega-lib?noext\",\n",
       "      \"vega-lite\": \"https://cdn.jsdelivr.net/npm/vega-lite@5.16.3?noext\",\n",
       "      \"vega-embed\": \"https://cdn.jsdelivr.net/npm/vega-embed@6?noext\",\n",
       "    };\n",
       "\n",
       "    function maybeLoadScript(lib, version) {\n",
       "      var key = `${lib.replace(\"-\", \"\")}_version`;\n",
       "      return (VEGA_DEBUG[key] == version) ?\n",
       "        Promise.resolve(paths[lib]) :\n",
       "        new Promise(function(resolve, reject) {\n",
       "          var s = document.createElement('script');\n",
       "          document.getElementsByTagName(\"head\")[0].appendChild(s);\n",
       "          s.async = true;\n",
       "          s.onload = () => {\n",
       "            VEGA_DEBUG[key] = version;\n",
       "            return resolve(paths[lib]);\n",
       "          };\n",
       "          s.onerror = () => reject(`Error loading script: ${paths[lib]}`);\n",
       "          s.src = paths[lib];\n",
       "        });\n",
       "    }\n",
       "\n",
       "    function showError(err) {\n",
       "      outputDiv.innerHTML = `<div class=\"error\" style=\"color:red;\">${err}</div>`;\n",
       "      throw err;\n",
       "    }\n",
       "\n",
       "    function displayChart(vegaEmbed) {\n",
       "      vegaEmbed(outputDiv, spec, embedOpt)\n",
       "        .catch(err => showError(`Javascript Error: ${err.message}<br>This usually means there's a typo in your chart specification. See the javascript console for the full traceback.`));\n",
       "    }\n",
       "\n",
       "    if(typeof define === \"function\" && define.amd) {\n",
       "      requirejs.config({paths});\n",
       "      require([\"vega-embed\"], displayChart, err => showError(`Error loading script: ${err.message}`));\n",
       "    } else {\n",
       "      maybeLoadScript(\"vega\", \"5\")\n",
       "        .then(() => maybeLoadScript(\"vega-lite\", \"5.16.3\"))\n",
       "        .then(() => maybeLoadScript(\"vega-embed\", \"6\"))\n",
       "        .catch(showError)\n",
       "        .then(() => displayChart(vegaEmbed));\n",
       "    }\n",
       "  })({\"config\": {\"view\": {\"continuousWidth\": 300, \"continuousHeight\": 300}}, \"hconcat\": [{\"layer\": [{\"data\": {\"name\": \"data-68c5fc4a60be90d486fe214af3b8cb49\"}, \"mark\": {\"type\": \"line\", \"color\": \"black\", \"opacity\": 1, \"strokeWidth\": 1}, \"encoding\": {\"detail\": {\"field\": \"edge\", \"type\": \"quantitative\"}, \"x\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false, \"title\": \"\"}, \"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false, \"title\": \"\"}, \"field\": \"y\", \"type\": \"quantitative\"}}, \"name\": \"view_2\"}, {\"data\": {\"name\": \"data-9dd8291c49693efa9806d078391fedf1\"}, \"mark\": {\"type\": \"point\", \"opacity\": 1, \"strokeWidth\": 1.0}, \"encoding\": {\"fill\": {\"field\": \"louvain_partition\", \"type\": \"nominal\"}, \"size\": {\"field\": \"degree_centrality\", \"legend\": null, \"type\": \"quantitative\"}, \"tooltip\": [{\"field\": \"degree_centrality\", \"type\": \"quantitative\"}, {\"field\": \"betweenness_centrality\", \"type\": \"quantitative\"}], \"x\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false}, \"field\": \"x\", \"type\": \"quantitative\"}, \"y\": {\"axis\": {\"grid\": false, \"labels\": false, \"ticks\": false}, \"field\": \"y\", \"type\": \"quantitative\"}}}]}, {\"data\": {\"name\": \"data-9dd8291c49693efa9806d078391fedf1\"}, \"mark\": {\"type\": \"bar\"}, \"encoding\": {\"color\": {\"field\": \"louvain_partition\", \"legend\": null, \"type\": \"nominal\"}, \"x\": {\"field\": \"louvain_partition\", \"type\": \"nominal\"}, \"y\": {\"aggregate\": \"count\", \"type\": \"quantitative\"}}}], \"params\": [{\"name\": \"param_2\", \"select\": {\"type\": \"interval\", \"encodings\": [\"x\", \"y\"]}, \"bind\": \"scales\", \"views\": [\"view_2\"]}], \"$schema\": \"https://vega.github.io/schema/vega-lite/v5.16.3.json\", \"datasets\": {\"data-68c5fc4a60be90d486fe214af3b8cb49\": [{\"target\": 1, \"weight\": 4, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 1], \"edge\": 0, \"source\": 0}, {\"target\": 1, \"weight\": 4, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [0, 1], \"edge\": 0, \"source\": 0}, {\"target\": 2, \"weight\": 5, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 2], \"edge\": 1, \"source\": 0}, {\"target\": 2, \"weight\": 5, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [0, 2], \"edge\": 1, \"source\": 0}, {\"target\": 3, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 3], \"edge\": 2, \"source\": 0}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [0, 3], \"edge\": 2, \"source\": 0}, {\"target\": 4, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 4], \"edge\": 3, \"source\": 0}, {\"target\": 4, \"weight\": 3, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [0, 4], \"edge\": 3, \"source\": 0}, {\"target\": 5, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 5], \"edge\": 4, \"source\": 0}, {\"target\": 5, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [0, 5], \"edge\": 4, \"source\": 0}, {\"target\": 6, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 6], \"edge\": 5, \"source\": 0}, {\"target\": 6, \"weight\": 3, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [0, 6], \"edge\": 5, \"source\": 0}, {\"target\": 7, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 7], \"edge\": 6, \"source\": 0}, {\"target\": 7, \"weight\": 2, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [0, 7], \"edge\": 6, \"source\": 0}, {\"target\": 8, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 8], \"edge\": 7, \"source\": 0}, {\"target\": 8, \"weight\": 2, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [0, 8], \"edge\": 7, \"source\": 0}, {\"target\": 10, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 10], \"edge\": 8, \"source\": 0}, {\"target\": 10, \"weight\": 2, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [0, 10], \"edge\": 8, \"source\": 0}, {\"target\": 11, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 11], \"edge\": 9, \"source\": 0}, {\"target\": 11, \"weight\": 3, \"y\": 0.597088045182672, \"x\": 0.007541601265146356, \"pair\": [0, 11], \"edge\": 9, \"source\": 0}, {\"target\": 12, \"weight\": 1, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 12], \"edge\": 10, \"source\": 0}, {\"target\": 12, \"weight\": 1, \"y\": 0.3853633418034468, \"x\": 0.6506086687459723, \"pair\": [0, 12], \"edge\": 10, \"source\": 0}, {\"target\": 13, \"weight\": 3, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 13], \"edge\": 11, \"source\": 0}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [0, 13], \"edge\": 11, \"source\": 0}, {\"target\": 17, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 17], \"edge\": 12, \"source\": 0}, {\"target\": 17, \"weight\": 2, \"y\": 0.21975436759374803, \"x\": 0.6257949010031251, \"pair\": [0, 17], \"edge\": 12, \"source\": 0}, {\"target\": 19, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 19], \"edge\": 13, \"source\": 0}, {\"target\": 19, \"weight\": 2, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [0, 19], \"edge\": 13, \"source\": 0}, {\"target\": 21, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 21], \"edge\": 14, \"source\": 0}, {\"target\": 21, \"weight\": 2, \"y\": 0.3912476160151856, \"x\": 0.4663145131865174, \"pair\": [0, 21], \"edge\": 14, \"source\": 0}, {\"target\": 31, \"weight\": 2, \"y\": 0.3097536305922184, \"x\": 0.22503121822115288, \"pair\": [0, 31], \"edge\": 15, \"source\": 0}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [0, 31], \"edge\": 15, \"source\": 0}, {\"target\": 2, \"weight\": 6, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 2], \"edge\": 16, \"source\": 1}, {\"target\": 2, \"weight\": 6, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [1, 2], \"edge\": 16, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 3], \"edge\": 17, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [1, 3], \"edge\": 17, \"source\": 1}, {\"target\": 7, \"weight\": 4, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 7], \"edge\": 18, \"source\": 1}, {\"target\": 7, \"weight\": 4, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [1, 7], \"edge\": 18, \"source\": 1}, {\"target\": 13, \"weight\": 5, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 13], \"edge\": 19, \"source\": 1}, {\"target\": 13, \"weight\": 5, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [1, 13], \"edge\": 19, \"source\": 1}, {\"target\": 17, \"weight\": 1, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 17], \"edge\": 20, \"source\": 1}, {\"target\": 17, \"weight\": 1, \"y\": 0.21975436759374803, \"x\": 0.6257949010031251, \"pair\": [1, 17], \"edge\": 20, \"source\": 1}, {\"target\": 19, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 19], \"edge\": 21, \"source\": 1}, {\"target\": 19, \"weight\": 2, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [1, 19], \"edge\": 21, \"source\": 1}, {\"target\": 21, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 21], \"edge\": 22, \"source\": 1}, {\"target\": 21, \"weight\": 2, \"y\": 0.3912476160151856, \"x\": 0.4663145131865174, \"pair\": [1, 21], \"edge\": 22, \"source\": 1}, {\"target\": 30, \"weight\": 2, \"y\": 0.13856825943954948, \"x\": 0.22132660602478407, \"pair\": [1, 30], \"edge\": 23, \"source\": 1}, {\"target\": 30, \"weight\": 2, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [1, 30], \"edge\": 23, \"source\": 1}, {\"target\": 3, \"weight\": 3, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 3], \"edge\": 24, \"source\": 2}, {\"target\": 3, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [2, 3], \"edge\": 24, \"source\": 2}, {\"target\": 7, \"weight\": 4, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 7], \"edge\": 25, \"source\": 2}, {\"target\": 7, \"weight\": 4, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [2, 7], \"edge\": 25, \"source\": 2}, {\"target\": 8, \"weight\": 5, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 8], \"edge\": 26, \"source\": 2}, {\"target\": 8, \"weight\": 5, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [2, 8], \"edge\": 26, \"source\": 2}, {\"target\": 9, \"weight\": 1, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 9], \"edge\": 27, \"source\": 2}, {\"target\": 9, \"weight\": 1, \"y\": -0.5999087969351852, \"x\": -0.04073657086420715, \"pair\": [2, 9], \"edge\": 27, \"source\": 2}, {\"target\": 13, \"weight\": 3, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 13], \"edge\": 28, \"source\": 2}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [2, 13], \"edge\": 28, \"source\": 2}, {\"target\": 27, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 27], \"edge\": 29, \"source\": 2}, {\"target\": 27, \"weight\": 2, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [2, 27], \"edge\": 29, \"source\": 2}, {\"target\": 28, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 28], \"edge\": 30, \"source\": 2}, {\"target\": 28, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [2, 28], \"edge\": 30, \"source\": 2}, {\"target\": 32, \"weight\": 2, \"y\": -0.026723682993196735, \"x\": 0.11712199222035433, \"pair\": [2, 32], \"edge\": 31, \"source\": 2}, {\"target\": 32, \"weight\": 2, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [2, 32], \"edge\": 31, \"source\": 2}, {\"target\": 7, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 7], \"edge\": 32, \"source\": 3}, {\"target\": 7, \"weight\": 3, \"y\": 0.07616207795941865, \"x\": 0.36705456566283584, \"pair\": [3, 7], \"edge\": 32, \"source\": 3}, {\"target\": 12, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 12], \"edge\": 33, \"source\": 3}, {\"target\": 12, \"weight\": 3, \"y\": 0.3853633418034468, \"x\": 0.6506086687459723, \"pair\": [3, 12], \"edge\": 33, \"source\": 3}, {\"target\": 13, \"weight\": 3, \"y\": 0.20274363377868745, \"x\": 0.3613046705731705, \"pair\": [3, 13], \"edge\": 34, \"source\": 3}, {\"target\": 13, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [3, 13], \"edge\": 34, \"source\": 3}, {\"target\": 6, \"weight\": 2, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [4, 6], \"edge\": 35, \"source\": 4}, {\"target\": 6, \"weight\": 2, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [4, 6], \"edge\": 35, \"source\": 4}, {\"target\": 10, \"weight\": 3, \"y\": 0.6247825385327798, \"x\": 0.26353120038140965, \"pair\": [4, 10], \"edge\": 36, \"source\": 4}, {\"target\": 10, \"weight\": 3, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [4, 10], \"edge\": 36, \"source\": 4}, {\"target\": 6, \"weight\": 5, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 6], \"edge\": 37, \"source\": 5}, {\"target\": 6, \"weight\": 5, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [5, 6], \"edge\": 37, \"source\": 5}, {\"target\": 10, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 10], \"edge\": 38, \"source\": 5}, {\"target\": 10, \"weight\": 3, \"y\": 0.7550530935308861, \"x\": 0.178048351734203, \"pair\": [5, 10], \"edge\": 38, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 0.760957365547892, \"x\": 0.3348488047284244, \"pair\": [5, 16], \"edge\": 39, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 1.0, \"x\": 0.46353845653581993, \"pair\": [5, 16], \"edge\": 39, \"source\": 5}, {\"target\": 16, \"weight\": 3, \"y\": 0.7112806407924765, \"x\": 0.42635535723687124, \"pair\": [6, 16], \"edge\": 40, \"source\": 6}, {\"target\": 16, \"weight\": 3, \"y\": 1.0, \"x\": 0.46353845653581993, \"pair\": [6, 16], \"edge\": 40, \"source\": 6}, {\"target\": 30, \"weight\": 3, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 30], \"edge\": 41, \"source\": 8}, {\"target\": 30, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [8, 30], \"edge\": 41, \"source\": 8}, {\"target\": 32, \"weight\": 3, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 32], \"edge\": 42, \"source\": 8}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [8, 32], \"edge\": 42, \"source\": 8}, {\"target\": 33, \"weight\": 4, \"y\": -0.030123270746296296, \"x\": -0.09506079209675, \"pair\": [8, 33], \"edge\": 43, \"source\": 8}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [8, 33], \"edge\": 43, \"source\": 8}, {\"target\": 33, \"weight\": 2, \"y\": -0.5999087969351852, \"x\": -0.04073657086420715, \"pair\": [9, 33], \"edge\": 44, \"source\": 9}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [9, 33], \"edge\": 44, \"source\": 9}, {\"target\": 33, \"weight\": 3, \"y\": 0.09163458388764495, \"x\": 0.062256995703755307, \"pair\": [13, 33], \"edge\": 45, \"source\": 13}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [13, 33], \"edge\": 45, \"source\": 13}, {\"target\": 32, \"weight\": 3, \"y\": -0.07645960350272783, \"x\": -0.5860519476705567, \"pair\": [14, 32], \"edge\": 46, \"source\": 14}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [14, 32], \"edge\": 46, \"source\": 14}, {\"target\": 33, \"weight\": 2, \"y\": -0.07645960350272783, \"x\": -0.5860519476705567, \"pair\": [14, 33], \"edge\": 47, \"source\": 14}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [14, 33], \"edge\": 47, \"source\": 14}, {\"target\": 32, \"weight\": 3, \"y\": -0.17961314191255023, \"x\": -0.5103533762523353, \"pair\": [15, 32], \"edge\": 48, \"source\": 15}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [15, 32], \"edge\": 48, \"source\": 15}, {\"target\": 33, \"weight\": 4, \"y\": -0.17961314191255023, \"x\": -0.5103533762523353, \"pair\": [15, 33], \"edge\": 49, \"source\": 15}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [15, 33], \"edge\": 49, \"source\": 15}, {\"target\": 32, \"weight\": 1, \"y\": -0.5082714987382673, \"x\": -0.6545540664183989, \"pair\": [18, 32], \"edge\": 50, \"source\": 18}, {\"target\": 32, \"weight\": 1, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [18, 32], \"edge\": 50, \"source\": 18}, {\"target\": 33, \"weight\": 2, \"y\": -0.5082714987382673, \"x\": -0.6545540664183989, \"pair\": [18, 33], \"edge\": 51, \"source\": 18}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [18, 33], \"edge\": 51, \"source\": 18}, {\"target\": 33, \"weight\": 1, \"y\": -0.05620242556660995, \"x\": 0.27260396115187274, \"pair\": [19, 33], \"edge\": 52, \"source\": 19}, {\"target\": 33, \"weight\": 1, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [19, 33], \"edge\": 52, \"source\": 19}, {\"target\": 32, \"weight\": 3, \"y\": -0.6060332323179136, \"x\": -0.4421316341277511, \"pair\": [20, 32], \"edge\": 53, \"source\": 20}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [20, 32], \"edge\": 53, \"source\": 20}, {\"target\": 33, \"weight\": 1, \"y\": -0.6060332323179136, \"x\": -0.4421316341277511, \"pair\": [20, 33], \"edge\": 54, \"source\": 20}, {\"target\": 33, \"weight\": 1, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [20, 33], \"edge\": 54, \"source\": 20}, {\"target\": 32, \"weight\": 2, \"y\": -0.4752818193287819, \"x\": -0.4901750606421922, \"pair\": [22, 32], \"edge\": 55, \"source\": 22}, {\"target\": 32, \"weight\": 2, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [22, 32], \"edge\": 55, \"source\": 22}, {\"target\": 33, \"weight\": 3, \"y\": -0.4752818193287819, \"x\": -0.4901750606421922, \"pair\": [22, 33], \"edge\": 56, \"source\": 22}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [22, 33], \"edge\": 56, \"source\": 22}, {\"target\": 25, \"weight\": 5, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 25], \"edge\": 57, \"source\": 23}, {\"target\": 25, \"weight\": 5, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [23, 25], \"edge\": 57, \"source\": 23}, {\"target\": 27, \"weight\": 4, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 27], \"edge\": 58, \"source\": 23}, {\"target\": 27, \"weight\": 4, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [23, 27], \"edge\": 58, \"source\": 23}, {\"target\": 29, \"weight\": 3, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 29], \"edge\": 59, \"source\": 23}, {\"target\": 29, \"weight\": 3, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [23, 29], \"edge\": 59, \"source\": 23}, {\"target\": 32, \"weight\": 5, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 32], \"edge\": 60, \"source\": 23}, {\"target\": 32, \"weight\": 5, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [23, 32], \"edge\": 60, \"source\": 23}, {\"target\": 33, \"weight\": 4, \"y\": -0.44048234585790097, \"x\": -0.2700696987833476, \"pair\": [23, 33], \"edge\": 61, \"source\": 23}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [23, 33], \"edge\": 61, \"source\": 23}, {\"target\": 25, \"weight\": 2, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 25], \"edge\": 62, \"source\": 24}, {\"target\": 25, \"weight\": 2, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [24, 25], \"edge\": 62, \"source\": 24}, {\"target\": 27, \"weight\": 3, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 27], \"edge\": 63, \"source\": 24}, {\"target\": 27, \"weight\": 3, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [24, 27], \"edge\": 63, \"source\": 24}, {\"target\": 31, \"weight\": 2, \"y\": -0.6089417486546188, \"x\": 0.11368659554436691, \"pair\": [24, 31], \"edge\": 64, \"source\": 24}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [24, 31], \"edge\": 64, \"source\": 24}, {\"target\": 31, \"weight\": 7, \"y\": -0.49977953574607614, \"x\": -0.08287243910457731, \"pair\": [25, 31], \"edge\": 65, \"source\": 25}, {\"target\": 31, \"weight\": 7, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [25, 31], \"edge\": 65, \"source\": 25}, {\"target\": 29, \"weight\": 4, \"y\": -0.31531373964572107, \"x\": -0.715204123907239, \"pair\": [26, 29], \"edge\": 66, \"source\": 26}, {\"target\": 29, \"weight\": 4, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [26, 29], \"edge\": 66, \"source\": 26}, {\"target\": 33, \"weight\": 2, \"y\": -0.31531373964572107, \"x\": -0.715204123907239, \"pair\": [26, 33], \"edge\": 67, \"source\": 26}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [26, 33], \"edge\": 67, \"source\": 26}, {\"target\": 33, \"weight\": 4, \"y\": -0.3938248407142819, \"x\": -0.05031006926735855, \"pair\": [27, 33], \"edge\": 68, \"source\": 27}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [27, 33], \"edge\": 68, \"source\": 27}, {\"target\": 31, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [28, 31], \"edge\": 69, \"source\": 28}, {\"target\": 31, \"weight\": 2, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [28, 31], \"edge\": 69, \"source\": 28}, {\"target\": 33, \"weight\": 2, \"y\": -0.3218466214147138, \"x\": 0.1326826127027591, \"pair\": [28, 33], \"edge\": 70, \"source\": 28}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [28, 33], \"edge\": 70, \"source\": 28}, {\"target\": 32, \"weight\": 4, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [29, 32], \"edge\": 71, \"source\": 29}, {\"target\": 32, \"weight\": 4, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [29, 32], \"edge\": 71, \"source\": 29}, {\"target\": 33, \"weight\": 2, \"y\": -0.3438316635346198, \"x\": -0.5228689570526862, \"pair\": [29, 33], \"edge\": 72, \"source\": 29}, {\"target\": 33, \"weight\": 2, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [29, 33], \"edge\": 72, \"source\": 29}, {\"target\": 32, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [30, 32], \"edge\": 73, \"source\": 30}, {\"target\": 32, \"weight\": 3, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [30, 32], \"edge\": 73, \"source\": 30}, {\"target\": 33, \"weight\": 3, \"y\": -0.00897933918312558, \"x\": -0.2085724152806764, \"pair\": [30, 33], \"edge\": 74, \"source\": 30}, {\"target\": 33, \"weight\": 3, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [30, 33], \"edge\": 74, \"source\": 30}, {\"target\": 32, \"weight\": 4, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [31, 32], \"edge\": 75, \"source\": 31}, {\"target\": 32, \"weight\": 4, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [31, 32], \"edge\": 75, \"source\": 31}, {\"target\": 33, \"weight\": 4, \"y\": -0.26271885859567407, \"x\": -0.03777649743004404, \"pair\": [31, 33], \"edge\": 76, \"source\": 31}, {\"target\": 33, \"weight\": 4, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [31, 33], \"edge\": 76, \"source\": 31}, {\"target\": 33, \"weight\": 5, \"y\": -0.25186155455325504, \"x\": -0.32679233890906567, \"pair\": [32, 33], \"edge\": 77, \"source\": 32}, {\"target\": 33, \"weight\": 5, \"y\": -0.2581914747150901, \"x\": -0.25612108481535745, \"pair\": [32, 33], \"edge\": 77, \"source\": 32}], \"data-9dd8291c49693efa9806d078391fedf1\": [{\"y\": 0.3097536305922184, \"degree_centrality\": 0.48484848484848486, \"x\": 0.22503121822115288, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.43763528138528146}, {\"y\": 0.13856825943954948, \"degree_centrality\": 0.2727272727272727, \"x\": 0.22132660602478407, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.053936688311688304}, {\"y\": -0.026723682993196735, \"degree_centrality\": 0.30303030303030304, \"x\": 0.11712199222035433, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.14365680615680618}, {\"y\": 0.20274363377868745, \"degree_centrality\": 0.18181818181818182, \"x\": 0.3613046705731705, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.011909271284271283}, {\"y\": 0.6247825385327798, \"degree_centrality\": 0.09090909090909091, \"x\": 0.26353120038140965, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0006313131313131313}, {\"y\": 0.760957365547892, \"degree_centrality\": 0.12121212121212122, \"x\": 0.3348488047284244, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.02998737373737374}, {\"y\": 0.7112806407924765, \"degree_centrality\": 0.12121212121212122, \"x\": 0.42635535723687124, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.029987373737373736}, {\"y\": 0.07616207795941865, \"degree_centrality\": 0.12121212121212122, \"x\": 0.36705456566283584, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.0}, {\"y\": -0.030123270746296296, \"degree_centrality\": 0.15151515151515152, \"x\": -0.09506079209675, \"club\": \"Mr. Hi\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.05592682780182781}, {\"y\": -0.5999087969351852, \"degree_centrality\": 0.06060606060606061, \"x\": -0.04073657086420715, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0008477633477633478}, {\"y\": 0.7550530935308861, \"degree_centrality\": 0.09090909090909091, \"x\": 0.178048351734203, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0006313131313131313}, {\"y\": 0.597088045182672, \"degree_centrality\": 0.030303030303030304, \"x\": 0.007541601265146356, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": 0.3853633418034468, \"degree_centrality\": 0.06060606060606061, \"x\": 0.6506086687459723, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.0}, {\"y\": 0.09163458388764495, \"degree_centrality\": 0.15151515151515152, \"x\": 0.062256995703755307, \"club\": \"Mr. Hi\", \"louvain_partition\": \"0\", \"betweenness_centrality\": 0.04586339586339586}, {\"y\": -0.07645960350272783, \"degree_centrality\": 0.06060606060606061, \"x\": -0.5860519476705567, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.17961314191255023, \"degree_centrality\": 0.06060606060606061, \"x\": -0.5103533762523353, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": 1.0, \"degree_centrality\": 0.06060606060606061, \"x\": 0.46353845653581993, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": 0.21975436759374803, \"degree_centrality\": 0.06060606060606061, \"x\": 0.6257949010031251, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": -0.5082714987382673, \"degree_centrality\": 0.06060606060606061, \"x\": -0.6545540664183989, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.05620242556660995, \"degree_centrality\": 0.09090909090909091, \"x\": 0.27260396115187274, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.03247504810004811}, {\"y\": -0.6060332323179136, \"degree_centrality\": 0.06060606060606061, \"x\": -0.4421316341277511, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": 0.3912476160151856, \"degree_centrality\": 0.06060606060606061, \"x\": 0.4663145131865174, \"club\": \"Mr. Hi\", \"louvain_partition\": \"1\", \"betweenness_centrality\": 0.0}, {\"y\": -0.4752818193287819, \"degree_centrality\": 0.06060606060606061, \"x\": -0.4901750606421922, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.44048234585790097, \"degree_centrality\": 0.15151515151515152, \"x\": -0.2700696987833476, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.017613636363636363}, {\"y\": -0.6089417486546188, \"degree_centrality\": 0.09090909090909091, \"x\": 0.11368659554436691, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0022095959595959595}, {\"y\": -0.49977953574607614, \"degree_centrality\": 0.09090909090909091, \"x\": -0.08287243910457731, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0038404882154882154}, {\"y\": -0.31531373964572107, \"degree_centrality\": 0.06060606060606061, \"x\": -0.715204123907239, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0}, {\"y\": -0.3938248407142819, \"degree_centrality\": 0.12121212121212122, \"x\": -0.05031006926735855, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.02233345358345358}, {\"y\": -0.3218466214147138, \"degree_centrality\": 0.09090909090909091, \"x\": 0.1326826127027591, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.0017947330447330447}, {\"y\": -0.3438316635346198, \"degree_centrality\": 0.12121212121212122, \"x\": -0.5228689570526862, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.0029220779220779218}, {\"y\": -0.00897933918312558, \"degree_centrality\": 0.12121212121212122, \"x\": -0.2085724152806764, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.014411976911976909}, {\"y\": -0.26271885859567407, \"degree_centrality\": 0.18181818181818182, \"x\": -0.03777649743004404, \"club\": \"Officer\", \"louvain_partition\": \"2\", \"betweenness_centrality\": 0.13827561327561325}, {\"y\": -0.25186155455325504, \"degree_centrality\": 0.36363636363636365, \"x\": -0.32679233890906567, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.145247113997114}, {\"y\": -0.2581914747150901, \"degree_centrality\": 0.5151515151515151, \"x\": -0.25612108481535745, \"club\": \"Officer\", \"louvain_partition\": \"3\", \"betweenness_centrality\": 0.30407497594997596}]}}, {\"mode\": \"vega-lite\"});\n",
       "</script>"
      ],
      "text/plain": [
       "alt.HConcatChart(...)"
      ]
     },
     "execution_count": 6,
     "metadata": {},
     "output_type": "execute_result"
    }
   ],
   "source": [
    "nodes = viz.layer[1] # Get the correct part of your previous visualization\n",
    "\n",
    "# Create a bar chart based on nodes data\n",
    "bars = alt.Chart(nodes.data).mark_bar().encode(\n",
    "    x=alt.X('louvain_partition'),\n",
    "    y=alt.Y('count()'),\n",
    "    color=alt.Color('louvain_partition').legend(None),\n",
    ")\n",
    "\n",
    "# Place interactive network and bar chart side by side\n",
    "viz.interactive() | bars"
   ]
  }
 ],
 "metadata": {
  "kernelspec": {
   "display_name": "Python 3 (ipykernel)",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.9.6"
  }
 },
 "nbformat": 4,
 "nbformat_minor": 5
}
